/*----------------------------------------------------------------------
"Debugging Applications" (Microsoft Press)
Copyright (c) 1997-2000 John Robbins -- All rights reserved.
----------------------------------------------------------------------*/

#include "PCH.h"
#include "DebugInterface.h"
#include "Dll.h"
#include "Resource.h"
#include "DebuggeeInfo.h"
#include "ProcessDebugEvents.h"
#include "CreateDebugSynchEvents.h"
#include "ModuleNotification.h"

/*//////////////////////////////////////////////////////////////////////
                     Typedefs, Constants, & Defines
//////////////////////////////////////////////////////////////////////*/
// The package of parameters passed to the debugging thread.
typedef struct tag_THREADPARAMS
{
    LPCTSTR          szDebuggee ;
    LPCTSTR          szCmdLine  ;
    LPDWORD          lpPID      ;
    CDebugBaseUser * pUserClass ;
} THREADPARAMS , * PTHREADPARAMS ;

/*//////////////////////////////////////////////////////////////////////
                     File Scope Function Prototypes
//////////////////////////////////////////////////////////////////////*/
// The debug thread itself.
static void DebugThread ( LPVOID lpVoid ) ;

/*//////////////////////////////////////////////////////////////////////
                     !!IMPLEMENTATION BEGINS HERE!!
//////////////////////////////////////////////////////////////////////*/
HANDLE DEBUGINTERFACE_DLLINTERFACE __stdcall
    StartDebugging ( LPCTSTR          szDebuggee        ,
                     LPCTSTR          szCmdLine         ,
                     LPDWORD          lpPID             ,
                     CDebugBaseUser * pUserClass        ,
                     LPHANDLE         lpDebugSyncEvents  )
{
    // Assert the parameters.
    ASSERT ( FALSE == IsBadStringPtr ( szDebuggee , MAX_PATH ) ) ;
    ASSERT ( FALSE == IsBadStringPtr ( szCmdLine , MAX_PATH ) ) ;
    ASSERT ( FALSE == IsBadWritePtr ( lpPID , sizeof ( DWORD ) ) ) ;
    ASSERT ( FALSE == IsBadReadPtr ( pUserClass ,
                                     sizeof ( CDebugBaseUser * ) ) ) ;
    ASSERT ( FALSE == IsBadWritePtr ( lpDebugSyncEvents ,
                                      sizeof ( HANDLE ) *
                                            NUM_DEBUGEVENTS ) ) ;
    // Check them all for real.
    if ( ( TRUE == IsBadStringPtr ( szDebuggee , MAX_PATH )     )  ||
         ( TRUE == IsBadStringPtr ( szCmdLine , MAX_PATH )      )  ||
         ( TRUE == IsBadWritePtr ( lpPID , sizeof ( DWORD )   ) )  ||
         ( TRUE == IsBadReadPtr ( pUserClass ,
                                  sizeof ( CDebugBaseUser * ) ) )  ||
         ( TRUE == IsBadWritePtr ( lpDebugSyncEvents ,
                                   sizeof ( HANDLE ) *
                                        NUM_DEBUGEVENTS )     )       )
    {
        SetLastError ( ERROR_INVALID_PARAMETER ) ;
        return ( INVALID_HANDLE_VALUE ) ;
    }

    // The handle of the startup acknowledgment that this function
    // will wait on until the debug thread gets started
    HANDLE   hStartAck ;
    // The string used for the startup acknowledgment event
    TCHAR    szStartAck [ MAX_PATH ] ;

    // Load up the string for startup acknowledgment.
    if ( 0 == LoadString ( GetDllHandle ( )      ,
                           IDS_DBGEVENTINIT      ,
                           szStartAck            ,
                           sizeof ( szStartAck )  ) )
    {
        ASSERT ( !"LoadString IDS_DBGEVENTINIT failed!" ) ;
        return ( INVALID_HANDLE_VALUE ) ;
    }

    // Create the startup acknowledgment event.
    hStartAck = CreateEvent ( NULL   ,     // Default security
                              TRUE   ,     // Manual-reset event
                              FALSE  ,     // Initial state=Not signaled
                              szStartAck ) ; // Event name
    ASSERT ( FALSE != hStartAck ) ;
    if ( FALSE == hStartAck )
    {
        TRACE ( "StartDebugging : Unable to create Start Ack event\n" ) ;
        return ( INVALID_HANDLE_VALUE ) ;
    }

    // Bundle up the parameters.
    THREADPARAMS stParams ;
    stParams.lpPID = lpPID ;
    stParams.pUserClass = pUserClass ;
    stParams.szDebuggee = szDebuggee ;
    stParams.szCmdLine  = szCmdLine  ;

    // The handle to the debug thread
    HANDLE hDbgThread ;

    // Try to create the thread.
    hDbgThread = (HANDLE)_beginthread ( DebugThread , 0 , &stParams ) ;
    ASSERT ( NULL != hDbgThread ) ;
    if ( NULL == hDbgThread )
    {
        TRACE ( "StartDebugging : _beginthread failed\n" ) ;
        VERIFY ( CloseHandle ( hStartAck ) ) ;
        return ( INVALID_HANDLE_VALUE ) ;
    }

    // Wait until the debug thread gets good and cranking.
    ::WaitForSingleObject ( hStartAck , INFINITE ) ;

    // Get rid of the acknowledgment handle.
    VERIFY ( CloseHandle ( hStartAck ) ) ;

    // Check that the debug thread is still running. If it isnt,
     // the debuggee probably couldnt get started.
    DWORD dwExitCode = ~STILL_ACTIVE ;
    if ( FALSE == GetExitCodeThread ( hDbgThread , &dwExitCode ) )
    {
        ASSERT ( !"GetExitCodeThread failed!" ) ;
        return ( INVALID_HANDLE_VALUE ) ;
    }
    ASSERT ( STILL_ACTIVE == dwExitCode ) ;
    if ( STILL_ACTIVE != dwExitCode )
    {
        TRACE ( "StartDebugging : GetExitCodeThread failed\n" ) ;
        return ( INVALID_HANDLE_VALUE ) ;
    }

    // Create the synchronization events so that the main thread can
    // tell the debug loop what to do.
    BOOL bCreateDbgSyncEvts =
                  CreateDebugSyncEvents ( lpDebugSyncEvents , *lpPID ) ;
    ASSERT ( TRUE == bCreateDbgSyncEvts ) ;
    if ( FALSE == bCreateDbgSyncEvts )
    {
        // This is a serious problem. I got the debug thread going, but
        // I was unable to create the synchronization events that the
        // user interface thread needs to control the debug thread. My
        // only option here is to punt. Ill kill the
        // debug thread and just return. I cant do much else.
        TRACE ( "StartDebugging : CreateDebugSyncEvents failed\n" ) ;
        VERIFY ( TerminateThread ( hDbgThread , (DWORD)-1 ) ) ;
        return ( INVALID_HANDLE_VALUE ) ;
    }
    // Life is good!
    return ( hDbgThread ) ;
}

BOOL DEBUGINTERFACE_DLLINTERFACE __stdcall
    StopDebugging ( LPHANDLE lpDebugSyncEvents )
{
    ASSERT ( FALSE ==
                IsBadWritePtr ( lpDebugSyncEvents ,
                                sizeof ( HANDLE ) * NUM_DEBUGEVENTS ) );
    if ( TRUE == IsBadWritePtr ( lpDebugSyncEvents ,
                                sizeof ( HANDLE ) * NUM_DEBUGEVENTS ) )
    {
        SetLastError ( ERROR_INVALID_PARAMETER ) ;
        return ( FALSE ) ;
    }

    // Signal the debug thread with the event name to close.
    VERIFY ( SetEvent ( lpDebugSyncEvents[ CLOSEDEBUGGER ] ) ) ;
    return ( TRUE ) ;
}

BOOL DEBUGINTERFACE_DLLINTERFACE __stdcall
    SuspendDebugging ( LPHANDLE lpDebugSyncEvents )
{
    ASSERT ( FALSE ==
                IsBadWritePtr ( lpDebugSyncEvents ,
                                sizeof ( HANDLE ) * NUM_DEBUGEVENTS ) );
    if ( TRUE == IsBadWritePtr ( lpDebugSyncEvents ,
                                sizeof ( HANDLE ) * NUM_DEBUGEVENTS ) )
    {
        SetLastError ( ERROR_INVALID_PARAMETER ) ;
        return ( FALSE ) ;
    }
    BOOL bSE = SetEvent ( lpDebugSyncEvents [ SUSPENDDEBUGGER ] ) ;
    ASSERT ( TRUE == bSE ) ;
    return ( bSE ) ;
}

BOOL DEBUGINTERFACE_DLLINTERFACE __stdcall
    ResumeDebugging ( LPHANDLE lpDebugSyncEvents )
{
    ASSERT ( FALSE ==
                IsBadWritePtr ( lpDebugSyncEvents ,
                                sizeof ( HANDLE ) * NUM_DEBUGEVENTS ) );
    if ( TRUE == IsBadWritePtr ( lpDebugSyncEvents ,
                                sizeof ( HANDLE ) * NUM_DEBUGEVENTS ) )
    {
        SetLastError ( ERROR_INVALID_PARAMETER ) ;
        return ( FALSE ) ;
    }
    BOOL bSE = SetEvent ( lpDebugSyncEvents [ RESUMEDEBUGGER ] ) ;
    ASSERT ( TRUE == bSE ) ;
    return ( bSE ) ;
}


// The happy debug thread itself!
static void DebugThread ( LPVOID lpVoid )
{
    // Take care of the parameters. No need to check as this function
    // will not get called unless everything lines up.
    PTHREADPARAMS pParams = (PTHREADPARAMS)lpVoid ;

    // The main thread notification functions.
    CDebugBaseUser * pUserClass = pParams->pUserClass ;

    // A string buffer...
    TCHAR szBuff [ MAX_PATH ] ;
    // An event handle..
    HANDLE hEvent  ;
    // All the synchronization events for this thread.
    HANDLE hDbgEvents[ NUM_DEBUGEVENTS ] ;
    // The debuggee name.
    TCHAR  szName [ MAX_PATH ];

    // Get the sync event string.
    if ( FALSE == LoadString ( GetDllHandle ( )  ,
                               IDS_DBGEVENTINIT  ,
                               szBuff            ,
                               sizeof ( szBuff )   ) )
    {
        ASSERT ( !"LoadString IDS_DBGEVENTINIT failed!" ) ;
        return ;
    }


    // Prepare to startup the debuggee.
    STARTUPINFO         si ;
    PROCESS_INFORMATION pi ;

    memset ( &si , NULL , sizeof ( STARTUPINFO ) ) ;
    memset ( &pi , NULL , sizeof ( PROCESS_INFORMATION ) ) ;

    // When starting, always show the app start cursor and show the
    // window normally.
    si.cb = sizeof ( STARTUPINFO ) ;
    si.dwFlags = STARTF_FORCEONFEEDBACK | STARTF_USESHOWWINDOW ;
    si.wShowWindow = SW_SHOWNORMAL ;

    // Crank up the debuggee!
    // DEBUG_ONLY_THIS_PROCESS is the magic flag that tells the OS
    // that this is going to be a debugger.  Also set CREATE_NEW_CONSOLE
    // so that any console applications get their own console window.
    BOOL bRetCreateProc =
        CreateProcess ( (LPTSTR)pParams->szDebuggee ,
                        (LPTSTR)pParams->szCmdLine  ,
                        NULL                        ,
                        NULL                        ,
                        FALSE                       ,
                        DEBUG_ONLY_THIS_PROCESS |
                            CREATE_NEW_CONSOLE      ,
                        NULL                        ,
                        NULL                        ,
                        &si                         ,
                        &pi                          ) ;
    ASSERT ( TRUE == bRetCreateProc ) ;

    // Set the process ID here as the main thread might get a boost
    // and look at it a little early.
    *pParams->lpPID = pi.dwProcessId ;

    // Copy the debuggee name as well.
    _tcscpy ( szName , pParams->szDebuggee ) ;

    // No matter if the debuggee was started or not, I still need to
    // clear out the initialization event.  The main thread is blocked
    // waiting on this.
    hEvent = OpenEvent ( EVENT_ALL_ACCESS ,
                         FALSE            ,
                         szBuff            ) ;
    ASSERT ( NULL != hEvent ) ;
    VERIFY ( SetEvent ( hEvent ) ) ;
    VERIFY ( CloseHandle ( hEvent ) ) ;

    // If the CreateProcess failed, leave as there is nothing to do.
    if ( FALSE == bRetCreateProc )
    {
        return ;
    }

    //Close the thread handle from CreateProcess.
    VERIFY ( CloseHandle ( pi.hThread  ) ) ;

    // Create all the debug syncronization events for this thread.
    BOOL bCreateDbgEvts =
        CreateDebugSyncEvents ( hDbgEvents ,  pi.dwProcessId ) ;
    ASSERT ( TRUE == bCreateDbgEvts ) ;
    if ( TRUE != bCreateDbgEvts )
    {
        return ;
    }

    // Reinitialize the debuggee info stuff.
    ReinitializeDebuggeeInfo ( ) ;

    // The actual debug events.
    DEBUG_EVENT de ;
    // Flag that tells us if the debuggee still running or not.
    BOOL bDebuggeeRunning = TRUE ;
    // Holds the index of the signaled event.
    int iIndex ;

    // Loop until the debuggee stops or I am told to stop.
    while ( TRUE == bDebuggeeRunning )
    {
        // Wait on my synchronization events.  Note that ACTIVEDEBUGGER
        // is always signaled so this function does not block in the
        // normal case.
        iIndex = WaitForMultipleObjects ( NUM_DEBUGEVENTS ,
                                          hDbgEvents      ,
                                          FALSE           ,
                                          INFINITE         ) ;
        switch ( iIndex )
        {
            // The highest priority item.
            case CLOSEDEBUGGER   :
                {
                    // The debuggee is still running, so shut it down.
                    VERIFY ( TerminateProcess ( pi.hProcess , 0 ) ) ;
                    VERIFY ( CloseHandle ( pi.hProcess ) ) ;
                    // This thread is finished.
                    bDebuggeeRunning = FALSE ;
                }
                break ;
            case SUSPENDDEBUGGER    :
                {
                    // Make the active debugger event non-signaled so I
                    // don't pop back in there.
                    VERIFY ( ResetEvent ( hDbgEvents [ACTIVEDEBUGGER]));
                    // Put the suspend event back to it's normal state.
                    // This causes the WaitForMultipleObjects at the
                    // top there to just block until resume is signaled.
                    VERIFY ( ResetEvent ( hDbgEvents[SUSPENDDEBUGGER]));
                }
                break ;
            case RESUMEDEBUGGER     :
                {
                    // Set the active debugger event to signaled, its
                    // normal state.
                    VERIFY ( SetEvent ( hDbgEvents[ACTIVEDEBUGGER] ) ) ;
                    // Turn off the reset event, its normal state.
                    VERIFY ( ResetEvent ( hDbgEvents[RESUMEDEBUGGER])) ;
                }
                break ;
            case ACTIVEDEBUGGER     :
                {
                    // Just a reminder.  Unless suspended, the active
                    // debugger state is always signaled so this part
                    // is always running.

                    // Wait a bit for a debugger event.  If none comes
                    // in, then I pop back to the WaitForMultipleObjects
                    // at the top of the loop.
                    if ( WaitForDebugEvent ( &de , (DWORD)100 ) )
                    {
                        // The debuggee info for this process.
                        LPDEBUGGEEINFO pData ;

                        // The create process is the first event ever
                        // seen so create a debuggee info class for it.
                        if ( CREATE_PROCESS_DEBUG_EVENT ==
                                                   de.dwDebugEventCode )
                        {
                            pData =
                                CreateDebuggeeInfo ( pUserClass ,
                                                     szName      ) ;
                        }
                        else
                        {
                            // The debuggee info has already been
                            // created.
                            pData = GetDebuggeeInfo ( de.dwProcessId ) ;
                        }

                        ASSERT ( NULL != pData ) ;

                        if ( WAIT_OBJECT_0 ==
                                WaitForSingleObject(
                                           hDbgEvents[CLOSEDEBUGGER],0))
                        {
                            continue ;
                        }

                        // Do the work to make everything fun.
                        DWORD dwContinueStatus =
                            ProcessDebugEvents ( pUserClass ,
                                                 pData      ,
                                                 &de         ) ;

                        // Do a quicky check of the close event.  If it
                        // has been signaled, I cannot pump load
                        // notifications because it will deadlock.
                        if ( WAIT_OBJECT_0 ==
                                WaitForSingleObject(
                                           hDbgEvents[CLOSEDEBUGGER],0))
                        {
                            continue ;
                        }

                        // If the processing returned zero, or pulsing
                        // the notification handler returned zero,
                        // something seriously bas has happened.  I need
                        // to manually signal the close event and stop
                        // the debug thread.
                        if ( ( 0 == dwContinueStatus            ) ||
                             ( 0 == PulseModuleNotification ( ) )   )
                        {
                            VERIFY(SetEvent(hDbgEvents[CLOSEDEBUGGER]));
                            continue ;
                        }

                        // Let the thread continue.
                        VERIFY ( ContinueDebugEvent ( de.dwProcessId   ,
                                                      de.dwThreadId    ,
                                                     dwContinueStatus));

                        // If this event is an EXIT_PROCESS, then there
                        // is a little bit of stuff to do to keep this
                        // thread straight.
                        if ( EXIT_PROCESS_DEBUG_EVENT ==
                                                   de.dwDebugEventCode )
                        {
                            // Clear the active debugger event.
                            VERIFY ( ResetEvent (
                                           hDbgEvents[ACTIVEDEBUGGER]));

                            // Drop out of the thread.
                            bDebuggeeRunning = FALSE ;
                        }
                    }
                    else
                    {
                        if ( 0 == PulseModuleNotification ( ) )
                        {
                            VERIFY(SetEvent(hDbgEvents[CLOSEDEBUGGER]));
                            continue ;
                        }
                    }
                }
                break ;
            default :
                ASSERT ( !"Invalid wait index!" ) ;
                break ;
        }

    }

    // Close all of the debugging handles that belong to
    // this thread.
    for ( int i = 0 ; i < NUM_DEBUGEVENTS ; i++ )
    {
        VERIFY ( CloseHandle ( hDbgEvents[ i ] ) ) ;
    }

    // Get rid of the debuggee info for the process this thread is
    // debugging.
    LPDEBUGGEEINFO pData = GetDebuggeeInfo ( pi.dwProcessId ) ;
    ASSERT ( NULL != pData ) ;
    VERIFY ( DestroyDebuggeeInfo ( pData ) ) ;

    // If there are any extra module notifications laying around, get
    // rid of them.
    ClearModuleNotification ( ) ;

    // Have a good day!
    TRACE ( "Debug Thread 0x%08X ending!\n" , GetCurrentThreadId ( ) );
}
